
/* THIS SAMPLE PROGRAM IS PROVIDED AS IS. THE SAMPLE PROGRAM AND ANY RESULTS
 * OBTAINED FROM IT ARE PROVIDED WITHOUT ANY WARRANTIES OR REPRESENTATIONS,
 * EXPRESS, IMPLIED OR STATUTORY. */

namespace Bcas.Samples.BCASModule
{
    /// <summary>
    /// Define sample code for the ASN.1 type 'BBCard' defined in the 'BCAS' ASN.1 module.
    /// </summary>
    public static class TBcas
    {
        public static void Main()
        {
            int failures = 0;
            Bcas.BerCodec codec = new Bcas.BerCodec();

            if (!EncodeDecodeAndPrint(codec, CreateMockObject(), "MockObject"))
            {
                failures++;
            }
            if (!EncodeDecodeAndPrint(codec, CreateMyCard(), "MyCard"))
            {
                failures++;
            }

            if (failures > 0)
            {
                System.Console.WriteLine(failures + " values failed.");
            }
            else
            {
                System.Console.WriteLine("All values encoded and decoded successfully.");
            }
        }

        /// <summary>
        /// Creates a mock sample value.
        /// </summary>
        public static Bcas.BCASModule.BBCard CreateMockObject()
        {
            Bcas.BCASModule.BBCard pdu = new Bcas.BCASModule.BBCard();
            pdu.Name = "name";
            pdu.Team = "team";
            pdu.Age = 1;
            pdu.Position = "position";
            pdu.Handedness = Bcas.BCASModule.BBCard.HandednessType.LeftHanded;
            pdu.BattingAverage = 0.0;

            return pdu;
        }

        /// <summary>
        /// Creates a value corresponding to the 'BCAS.myCard' ASN.1 value.
        /// </summary>
        public static Bcas.BCASModule.BBCard CreateMyCard()
        {
            Bcas.BCASModule.BBCard pdu = new Bcas.BCASModule.BBCard();
            pdu.Name = "Casey";
            pdu.Team = "Mudville Nine";
            pdu.Age = 32;
            pdu.Position = "left field";
            pdu.Handedness = Bcas.BCASModule.BBCard.HandednessType.Ambidextrous;
            pdu.BattingAverage = 0.250;

            return pdu;
        }

        /// <summary>
        /// This sample method demonstrates how to access the fields of the 'Bcas.BCASModule.BBCard' type.
        /// This method prints the fields of the object using the helper SampleTextWriter class.
        /// <summary>
        public static void PrintValue(Bcas.BCASModule.BBCard value, Oss.Asn1.SampleTextWriter stw)
        {
            stw.WriteBeginSequence();
            stw.WriteFieldName("name");
            stw.WriteString(value.Name);
            stw.WriteFieldName("team");
            stw.WriteString(value.Team);
            stw.WriteFieldName("age");
            stw.WriteInteger(value.Age);
            stw.WriteFieldName("position");
            stw.WriteString(value.Position);
            stw.WriteFieldName("handedness");
            switch (value.Handedness)
            {
                case Bcas.BCASModule.BBCard.HandednessType.LeftHanded:
                    stw.WriteString("left-handed");
                    break;
                case Bcas.BCASModule.BBCard.HandednessType.RightHanded:
                    stw.WriteString("right-handed");
                    break;
                case Bcas.BCASModule.BBCard.HandednessType.Ambidextrous:
                    stw.WriteString("ambidextrous");
                    break;
                default:
                    stw.WriteInteger((int)value.Handedness);
                    break;
            }
            stw.WriteFieldName("batting-average");
            stw.WriteDouble(value.BattingAverage);
            stw.WriteEndSequence();
        }

        /// <summary>
        /// This sample method demonstrates how to encode, decode and print the 'Bcas.BCASModule.BBCard' type.
        /// <summary>
        public static bool EncodeDecodeAndPrint(Oss.Asn1.BaseCodec codec, Bcas.BCASModule.BBCard value, string valueName)
        {
            System.Console.WriteLine("EncodeDecodeAndPrint {0}", valueName);

            // print the input PDU using the BaseType.ToString() method
            System.Console.WriteLine("Input PDU:");
            System.Console.WriteLine(value);

            System.IO.MemoryStream stream = new System.IO.MemoryStream();
            try
            {
                System.Console.WriteLine("Encoding {0} PDU using {1}Codec:", valueName, codec.EncodingRules);
                // encode the PDU to a stream
                codec.Encode(value, stream);

                // obtain encoded bytes
                byte[] encoding = stream.ToArray();
                System.Console.WriteLine("PDU successfully encoded, in " + encoding.Length + " bytes");

                // print encoded bytes
                System.Console.WriteLine("Encoded value:");
                if (codec is Oss.Asn1.TextCodec)
                {
                    System.Console.WriteLine(System.Text.Encoding.Default.GetString(encoding));
                }
                else
                {
                    Oss.Asn1.ValueNotationFormatter.Print(encoding, encoding.Length, System.Console.Out);
                }
            }
            catch (System.Exception ex)
            {
                System.Console.WriteLine("Encoding failed ({0}).", valueName);
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine();
                return false;
            }

            Bcas.BCASModule.BBCard decoded = new Bcas.BCASModule.BBCard();
            try
            {
                System.Console.WriteLine("Decoding {0} PDU using {1}Codec:", valueName, codec.EncodingRules);
                // decode from stream into the new PDU
                stream.Position = 0;
                codec.Decode(stream, decoded);
            }
            catch (System.Exception ex)
            {
                System.Console.WriteLine("Decoding failed ({0}).", valueName);
                System.Console.WriteLine(ex.Message);
                System.Console.WriteLine();
                return false;
            }

            // print the decoded value using sample PrintValue method
            System.Console.WriteLine("Decoded PDU:");
            PrintValue(decoded, new Oss.Asn1.SampleTextWriter(System.Console.Out));
            System.Console.WriteLine();

            return true;
        }
    }
}
